'use strict';

var dnsjs = require('../../dns-lint');
var cli = require('cli');
cli.parse({
//  'b': [ false, 'set source IP address (defaults to 0.0.0.0)', 'string' ]
  'class': [ 'c', 'class (defaults to IN)', 'string', 'IN' ]
, 'debug': [ 'false', 'more verbose output', 'boolean', false ]
//, 'insecure': [ false, 'turn off RaNDOm cAPS required for securing queries']
//, 'ipv4': [ '4', 'use ipv4 exclusively (defaults to false)', 'boolean', false ]
//, 'ipv6': [ '6', 'use ipv6 exclusively (defaults to false)', 'boolean', false ]
//, 'json': [ false, 'output results as json', 'string' ]
//, 'lint': [ false, 'attack (in the metaphorical sense) a nameserver with all sorts of queries to test for correct responses', 'string', false ]
, 'mdns': [ false, "Alias for setting defaults to -p 5353 @224.0.0.251 -t PTR -q _services._dns-sd._udp.local and waiting for multiple responses", 'boolean', false ]
, 'output': [ 'o', 'output prefix to use for writing query and response(s) to disk', 'file' ]
, 'port': [ 'p', 'port (defaults to 53 for dns and 5353 for mdns)', 'int' ]
//, 'serve': [ 's', 'path to json file with array of responses to issue for given queries', 'string' ]
, 'type': [ 't', 'type (defaults to ANY for dns and PTR for mdns)', 'string' ]
, 'query': [ 'q', 'a superfluous explicit option to set the query as a command line flag' ]
});

var PromiseA = require('bluebird');
var fs = PromiseA.promisifyAll(require('fs'));
var dgram = require('dgram');
var commonTypes = [ 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT' ];

function hexdump(ab) {
  var ui8 = new Uint8Array(ab);
  var bytecount = 0;
  var head = '        0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F';
  var trail;
  var str = [].slice.call(ui8).map(function (i) {
    var h = i.toString(16);
    if (h.length < 2) {
      h = '0' + h;
    }
    return h;
  }).join('').match(/.{1,2}/g).join(' ').match(/.{1,48}/g).map(function (str) {
    var lead = bytecount.toString(16);
    bytecount += 16;

    while (lead.length < 7) {
      lead = '0' + lead;
    }

    return lead + ' ' + str;
  }).join('\n');
  trail = ab.byteLength.toString(16);
  while (trail.length < 7) {
    trail = '0' + trail;
  }
  return head + '\n' + str + '\n' + trail;
}

cli.main(function (args, cli) {
  args.forEach(function (arg) {
    if (-1 !== commonTypes.indexOf(arg.toUpperCase())) {
      if (cli.type) {
        console.error("'type' was specified more than once");
        process.exit(1);
        return;
      }
      cli.type = cli.t = arg.toUpperCase();
    }

    if (/^@/.test(arg)) {
      if (cli.nameserver) {
        console.error("'@server' was specified more than once");
        process.exit(1);
        return;
      }
      cli.nameserver = cli.n = arg;
    }

    if (cli.query) {
      console.error("'@server' was specified more than once");
      process.exit(1);
      return;
    }
    cli.query = cli.q = arg;

  });

  if (cli.mdns) {
    if (!cli.type) {
      cli.type = cli.t = 'PTR';
    }
    if (!cli.port) {
      cli.port = cli.p = 5353;
    }
  }

  if (!cli.type) {
    cli.type = cli.t = 'ANY';
  }
  if (!cli.port) {
    cli.port = cli.p = 53;
  }
  if (!cli.class) {
    cli.class = cli.c = 'IN';
  }
  if (!cli.query) {
    console.error('');
    console.error('Usage:');
    console.error('dig.js [@server] [TYPE] [domain]');
    console.error('');
    console.error('Example:');
    console.error('dig.js daplie.com');
    console.error('');
    process.exit(1);
  }

  var query = {
    header: {
      id: require('crypto').randomBytes(2).readUInt16BE(0)
    , qr: 0
    , opcode: 0
    , aa: 0     // NA
    , tc: 0     // NA
    , rd: 1
    , ra: 0     // NA
    , rcode: 0  // NA
    }
  , question: [
      { name: cli.query
      , typeName: cli.type
      , className: cli.class
      }
    ]
  };
  var queryAb = dnsjs.DNSPacket.write(query);

  if (cli.debug) {
    console.log('');
    console.log('DNS Question:');
    console.log('');
    console.log(query);
    console.log('');
    console.log(hexdump(queryAb));
    console.log('');
    console.log(dnsjs.DNSPacket.parse(queryAb));
    console.log('');
  }




    process.exit(1);
  var server = dgram.createSocket({
    type: 'udp4'
  , reuseAddr: true
  });

  var handlers = {};
  handlers.onError = function (err) {
      console.error("error:", err.stack);
      server.close();
  };
  handlers.onMessage = function (buffer) {
    var path = require('path');
    var filename = type + '-' + count + '.mdns.bin';
    var fullpath = path.join('samples', filename);

    count += 1;

    fs.writeFileAsync(fullpath, buffer).then(function () {
      console.log('wrote ' + buffer.length + ' bytes to ' + fullpath);
    });
  };
  handlers.onListening = function () {
    /*jshint validthis:true*/
    var server = this;
    console.log(server.address());

    server.setBroadcast(true);
    server.addMembership('224.0.0.251');
  };


  server.on('error', handlers.onError);
  server.on('message', handlers.onMessage);
  server.on('listening', handlers.onListening);

  // 53 dns
  // 5353 mdns
  server.bind(5353);
});
